'use client'
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _tslib = require('../../../_virtual/_tslib.cjs');
var viem = require('viem');
var accounts = require('viem/accounts');
var ethereumCore = require('@dynamic-labs/ethereum-core');
var utils = require('@dynamic-labs/utils');
var walletConnectorCore = require('@dynamic-labs/wallet-connector-core');
var WalletConnectProvider = require('../WalletConnectProvider/WalletConnectProvider.cjs');

const WC_CURRENT_CHAIN_KEY = 'dynamic-wc2-current-chain';
class WalletConnectConnector extends ethereumCore.EthereumWalletConnector {
    constructor(opts) {
        super(opts);
        this.canConnectViaQrCode = true;
        this.isWalletConnect = true;
        this.canHandleMultipleConnections = false;
        this.name = opts.walletName;
        this.deepLinkPreference = opts.deepLinkPreference || 'native';
        const storedChainId = utils.StorageService.getItem(WC_CURRENT_CHAIN_KEY);
        if (storedChainId) {
            this.currentChainId = utils.parseIntSafe(storedChainId);
        }
        if (!opts.projectId) {
            throw new utils.DynamicError('WalletConnect project ID is required');
        }
        // set provider props generic to all wallets
        WalletConnectProvider.WalletConnectProvider.projectId = opts.projectId;
        WalletConnectProvider.WalletConnectProvider.enabledNetworks = opts.evmNetworks;
        WalletConnectProvider.WalletConnectProvider.preferredChains =
            opts.walletConnectPreferredChains || [];
        WalletConnectProvider.WalletConnectProvider.evmNetworkRpcMap = this.evmNetworkRpcMap();
    }
    init() {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] init called', {
                isInitialized: WalletConnectProvider.WalletConnectProvider.isInitialized,
                isInitializing: WalletConnectProvider.WalletConnectProvider.isInitializing,
            });
            // we should only init the provider once as soon as possible
            // the connection is established when a wallet is selected (with getAddress)
            if (WalletConnectProvider.WalletConnectProvider.isInitialized ||
                WalletConnectProvider.WalletConnectProvider.isInitializing) {
                walletConnectorCore.logger.debug('[WalletConnect] init - already initialized or initializing - skipping');
                return;
            }
            walletConnectorCore.logger.debug('[WalletConnect] init');
            this.walletConnectorEventsEmitter.emit('connectorInitStarted', 'walletconnect');
            try {
                yield WalletConnectProvider.WalletConnectProvider.init();
            }
            catch (error) {
                walletConnectorCore.logger.error('[WalletConnect] init - error', error);
                throw new utils.DynamicError('WalletConnectProvider failed to initialize');
            }
            this.setupWCEventListeners();
            this.walletConnectorEventsEmitter.emit('connectorInitCompleted', 'walletconnect');
        });
    }
    setupWCEventListeners() {
        walletConnectorCore.logger.debug('[WalletConnect] setupWCEventListeners');
        WalletConnectProvider.WalletConnectProvider.teardownEventListeners();
        WalletConnectProvider.WalletConnectProvider.setupEventListeners({
            onAccountChanged: (account) => {
                walletConnectorCore.logger.debug('[WalletConnect] onAccountChanged', { account });
                this.emit('accountChange', { accounts: [account] });
            },
            onChainChanged: (chainId) => {
                walletConnectorCore.logger.debug('[WalletConnect] onChainChange', { chainId });
                if (chainId === this.currentChainId) {
                    walletConnectorCore.logger.debug(`[WalletConnect] onChainChange - ignoring chainChanged event with same chain id as current chain id: ${chainId}`);
                    return;
                }
                this.currentChainId = chainId;
                this.emit('chainChange', { chain: String(chainId) });
            },
            onDisconnect: () => {
                walletConnectorCore.logger.debug('[WalletConnect] onDisconnect');
                this.endSession();
                this.emit('disconnect');
            },
        });
    }
    endSession() {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            walletConnectorCore.logger.debug('[WalletConnect] endSession');
            this.currentChainId = undefined;
            yield WalletConnectProvider.WalletConnectProvider.disconnect();
        });
    }
    getAddress(opts) {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            walletConnectorCore.logger.debug('[WalletConnect] getAddress', opts);
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnectConnector] getAddress', {
                inAppBrowserUrl: (_a = this.metadata) === null || _a === void 0 ? void 0 : _a.inAppBrowserUrl,
                isMobile: utils.isMobile(),
                mobileExperience: this.mobileExperience,
            });
            if (utils.isMobile() &&
                ((_b = this.metadata) === null || _b === void 0 ? void 0 : _b.inAppBrowserUrl) &&
                this.mobileExperience === 'in-app-browser') {
                const inAppBrowserCompiledTemplate = utils.template(this.metadata.inAppBrowserUrl);
                const { href } = utils.PlatformService.getUrl();
                const deepLink = inAppBrowserCompiledTemplate({
                    encodedDappURI: encodeURIComponent(href),
                });
                utils.PlatformService.openURL(deepLink);
                return;
            }
            const provider = yield WalletConnectProvider.WalletConnectProvider.awaitAndGetProvider();
            walletConnectorCore.logger.debug('[WalletConnect] getAddress - connecting to WalletConnect', {
                provider,
            });
            const addresses = yield WalletConnectProvider.WalletConnectProvider.connect({
                connectionOpts: opts,
                deepLinkPreference: this.deepLinkPreference,
                deepLinks: this.metadata.deepLinks,
            });
            walletConnectorCore.logger.debug('[WalletConnect] getAddress - connection result', addresses);
            const address = addresses === null || addresses === void 0 ? void 0 : addresses[0];
            return address;
        });
    }
    getWalletClient(chainId) {
        walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getWalletClient was called - chainId', chainId);
        const provider = WalletConnectProvider.WalletConnectProvider.getProvider();
        if (!provider) {
            walletConnectorCore.logger.debug('[WalletConnect] getWalletClient - provider is not initialized');
            throw new utils.DynamicError('WalletConnectProvider is not initialized');
        }
        const walletClient = viem.createWalletClient({
            account: this.getActiveAccount(),
            chain: ethereumCore.chainsMap[chainId !== null && chainId !== void 0 ? chainId : String(this.currentChainId)],
            transport: viem.custom({
                request: (args) => {
                    this.deepLinkIfApplicable(args.method);
                    return provider.request(args).catch(ethereumCore.normalizeRpcError);
                },
            }, this.providersConfig.httpTransportConfig),
        });
        return walletClient;
    }
    deepLinkIfApplicable(method) {
        const methodsThatRequireDeepLink = [
            'personal_sign',
            'eth_sendTransaction',
            'eth_signTypedData_v4',
        ];
        const deepLink = this.getDeepLink();
        if (utils.isMobile() && deepLink && methodsThatRequireDeepLink.includes(method)) {
            utils.PlatformService.openURL(deepLink);
        }
    }
    signMessage(messageToSign) {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] signMessage', messageToSign);
            const activeAccount = this.getActiveAccount();
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] signMessage - activeAccount', activeAccount);
            if (!activeAccount) {
                return;
            }
            const walletClient = yield this.getWalletClient();
            return walletClient.signMessage({
                account: activeAccount,
                message: messageToSign,
            });
        });
    }
    getConnectedAccounts() {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            const activeAccount = this.getActiveAccount();
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getConnectedAccounts - activeAccount', activeAccount);
            return activeAccount ? [activeAccount.address] : [];
        });
    }
    getActiveAccount() {
        var _a;
        const provider = WalletConnectProvider.WalletConnectProvider.getProvider();
        const connectedAccount = (_a = provider === null || provider === void 0 ? void 0 : provider.accounts) === null || _a === void 0 ? void 0 : _a[0];
        walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getActiveAccount - connectedAccount', connectedAccount);
        if (!connectedAccount) {
            return undefined;
        }
        return accounts.toAccount(connectedAccount);
    }
    get currentChainId() {
        const lsCurrentChain = utils.StorageService.getItem(WC_CURRENT_CHAIN_KEY);
        try {
            return lsCurrentChain ? utils.parseIntSafe(lsCurrentChain) : undefined;
        }
        catch (e) {
            walletConnectorCore.logger.debug('[WalletConnect] getCurrentChainId - error', e);
            return undefined;
        }
    }
    set currentChainId(value) {
        if (value) {
            utils.StorageService.setItem(WC_CURRENT_CHAIN_KEY, value.toString());
        }
        else {
            utils.StorageService.removeItem(WC_CURRENT_CHAIN_KEY);
        }
    }
    getActiveChain() {
        if (!this.currentChainId) {
            return undefined;
        }
        return ethereumCore.chainsMap[this.currentChainId];
    }
    getNetwork() {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getNetwork');
            const provider = yield WalletConnectProvider.WalletConnectProvider.awaitAndGetProvider();
            if (provider === null || provider === void 0 ? void 0 : provider.chainId) {
                const network = provider.chainId;
                this.currentChainId = network;
                walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getNetwork - provider network', network);
                return network;
            }
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getNetwork - no provider found, returning current chain id', {
                currentChainId: this.currentChainId,
            });
            return this.currentChainId;
        });
    }
    providerSwitchNetwork(_a) {
        const _super = Object.create(null, {
            providerSwitchNetwork: { get: () => super.providerSwitchNetwork }
        });
        return _tslib.__awaiter(this, arguments, void 0, function* ({ network, }) {
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] providerSwitchNetwork - network', {
                network,
                switchNetworkOnlyFromWallet: this.switchNetworkOnlyFromWallet,
            });
            const currentNetworkId = yield this.getNetwork();
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] providerSwitchNetwork - currentNetworkId', currentNetworkId);
            if (currentNetworkId && currentNetworkId === network.chainId) {
                return;
            }
            if (this.switchNetworkOnlyFromWallet) {
                throw new utils.DynamicError('Network switching is only supported through the wallet');
            }
            const walletClient = yield this.getWalletClient();
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] providerSwitchNetwork - will switch network');
            yield _super.providerSwitchNetwork.call(this, { network, provider: walletClient });
            this.currentChainId = network.chainId;
            walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] providerSwitchNetwork - switched network', network.chainId);
            this.emit('chainChange', { chain: String(network.chainId) });
        });
    }
    supportsNetworkSwitching() {
        return true;
    }
    getSupportedNetworks() {
        return _tslib.__awaiter(this, void 0, void 0, function* () {
            var _a;
            const provider = yield WalletConnectProvider.WalletConnectProvider.awaitAndGetProvider();
            if (!(provider === null || provider === void 0 ? void 0 : provider.session)) {
                return [];
            }
            const chains = [];
            // Some wallet (i.e ZenGo) use namespaces.account to list supported chains
            // while others use keys within the namespaces object
            Object.keys(provider === null || provider === void 0 ? void 0 : provider.session.namespaces).forEach((key) => {
                if (key.startsWith('eip155:')) {
                    chains.push(key.split(':')[1]);
                }
            });
            (_a = provider === null || provider === void 0 ? void 0 : provider.session.namespaces.eip155) === null || _a === void 0 ? void 0 : _a.accounts.forEach((account) => chains.push(account.split(':')[1]));
            return chains.length
                ? chains
                : this.evmNetworks.map((network) => network.chainId.toString());
        });
    }
    getDeepLink() {
        var _a;
        const provider = WalletConnectProvider.WalletConnectProvider.getProvider();
        walletConnectorCore.logger.debug('[WalletConnect] getDeepLink', {
            hasSession: Boolean(provider === null || provider === void 0 ? void 0 : provider.session),
            topic: (_a = provider === null || provider === void 0 ? void 0 : provider.session) === null || _a === void 0 ? void 0 : _a.topic,
            uri: provider === null || provider === void 0 ? void 0 : provider.signer.uri,
        });
        if (!(provider === null || provider === void 0 ? void 0 : provider.session)) {
            return;
        }
        const deepLink = walletConnectorCore.getDeepLink({
            deepLinks: this.metadata.deepLinks,
            mode: 'regular',
            preference: this.deepLinkPreference,
            uri: provider.signer.uri,
        });
        walletConnectorCore.logger.logVerboseTroubleshootingMessage('[WalletConnect] getDeepLink - deepLink', deepLink);
        if (!deepLink) {
            return;
        }
        // we need to include the session topic here because it helps the wallet
        // auto redirect back to the dapp after signing
        return `${deepLink}?sessionTopic=${provider.session.topic}`;
    }
    getConnectionUri() {
        return WalletConnectProvider.WalletConnectProvider.getConnectionUri();
    }
}

exports.WalletConnectConnector = WalletConnectConnector;
